"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.FormContextApi = exports.FormContext = void 0;
var _react = _interopRequireDefault(require("react"));
var _Form = _interopRequireDefault(require("../Form"));
var _helpers = require("../helpers");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : String(i); }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
const FormContext = exports.FormContext = /*#__PURE__*/_react.default.createContext({});
const FormContextApi = exports.FormContextApi = /*#__PURE__*/_react.default.createContext({});
class FormBuilder extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      isFieldsTouched: false,
      fields: {},
      invalidFields: new Set(''),
      valid: true,
      submitting: false,
      mirroredErrors: null
    };
    this.internalsOnChanges = new Map();
    this.formContextApiValue = {};
    this.getFields = fieldKey => {
      const {
        fields
      } = this.state;
      if (fieldKey) {
        return fields[fieldKey];
      }
      return fields;
    };
    this.setFields = (updates, options) => {
      const updatedFieldsName = Object.keys(updates);
      this.setState(state => {
        const nextFields = _objectSpread({}, state.fields);
        updatedFieldsName.forEach(fieldKey => {
          if (nextFields[fieldKey]) {
            const {
              validate,
              errorMessage
            } = nextFields[fieldKey];
            const {
              value
            } = updates[fieldKey];
            nextFields[fieldKey] = _objectSpread(_objectSpread({}, nextFields[fieldKey]), {}, {
              error: (0, _helpers.callValidateFunctions)({
                value,
                validate,
                errorMessage
              })
            }, updates[fieldKey]);
          }
        });
        return _objectSpread({
          fields: nextFields
        }, (0, _helpers.checkValidFieldsAndForm)(nextFields, state.invalidFields));
      }, () => {
        if ((options === null || options === void 0 ? void 0 : options.changeEvent) !== false) {
          this.triggerOnChangeEvents({
            name: updatedFieldsName
          });
        }
      });
    };
    this.setFieldsValue = (updates, options) => {
      const data = Object.keys(updates).reduce((acc, key) => {
        acc[key] = {
          value: updates[key]
        };
        return acc;
      }, {});
      this.setFields(data, options);
    };
    this.getFieldsValue = fieldKey => {
      const {
        fields
      } = this.state;
      if (fieldKey) {
        var _fields$fieldKey;
        return (_fields$fieldKey = fields[fieldKey]) === null || _fields$fieldKey === void 0 ? void 0 : _fields$fieldKey.value;
      }
      const values = {};
      Object.keys(fields).forEach(name => values[name] = fields[name].value);
      return values;
    };
    this.setInternalOnChanges = config => {
      this.internalsOnChanges.set(config, config);
    };
    this.unsetInternalOnChanges = config => {
      this.internalsOnChanges.delete(config);
    };
    this.getInitialValues = fieldKey => {
      var _this$props$initialVa;
      return fieldKey ? (_this$props$initialVa = this.props.initialValues) === null || _this$props$initialVa === void 0 ? void 0 : _this$props$initialVa[fieldKey] : this.props.initialValues;
    };
    this.onSubmit = event => {
      event && event.preventDefault && event.preventDefault();
      const {
        fields
      } = this.state;
      this.toggleSubmitting(true);
      const {
        fieldsToSubmit,
        fieldsWithError,
        validFormAfterValidateUntouchedFields
      } = (0, _helpers.checkUnTouchedFields)(fields);
      let mergedFieldsWithErrors = fieldsWithError;
      this.state.invalidFields.forEach(fieldKey => {
        if (!mergedFieldsWithErrors) {
          mergedFieldsWithErrors = {};
        }
        mergedFieldsWithErrors[fieldKey] = fields[fieldKey];
      });
      if (!validFormAfterValidateUntouchedFields) {
        this.setState(state => ({
          valid: validFormAfterValidateUntouchedFields,
          fields: _objectSpread(_objectSpread({}, state.fields), mergedFieldsWithErrors || {}),
          submitting: false
        }));
      }
      const promise = this.props.onSubmit(fieldsToSubmit, mergedFieldsWithErrors);
      if (promise) {
        return promise.finally(() => this.toggleSubmitting(false));
      }
      this.toggleSubmitting(false);
    };
    this.toggleSubmitting = submitting => this.setState({
      submitting
    });
    this.registerField = _ref => {
      let {
          name
        } = _ref,
        field = _objectWithoutProperties(_ref, ["name"]);
      const {
        initialValues
      } = this.props;
      const value = initialValues && name in initialValues ? initialValues[name] : field.value;
      this.setState(state => {
        const nextField = (0, _helpers.setExtraFieldProps)(this.state.mirroredExtraFieldsProps, _objectSpread(_objectSpread({
          name
        }, field), {}, {
          value,
          touched: false,
          error: this.props.errors ? this.props.errors[name] : null
        }));
        return {
          fields: _objectSpread(_objectSpread({}, state.fields), {}, {
            [name]: nextField
          })
        };
      });
    };
    this.unregister = _ref2 => {
      let {
        name
      } = _ref2;
      this.setState(state => {
        const _state$fields = state.fields,
          {
            [name]: removed
          } = _state$fields,
          restFields = _objectWithoutProperties(_state$fields, [name].map(_toPropertyKey));
        // clear non-existing fields from invalidFields state
        state.invalidFields.delete(removed.name);
        return {
          fields: _objectSpread({}, restFields),
          // replace with shouldValidation variable instead
          valid: true
        };
      });
    };
    this.onChange = _ref3 => {
      let {
          name
        } = _ref3,
        updates = _objectWithoutProperties(_ref3, ["name"]);
      this.setState(state => {
        const prevField = state.fields[name];
        const nextField = _objectSpread(_objectSpread(_objectSpread({}, prevField), updates), {}, {
          touched: true
        });
        const nextFields = _objectSpread(_objectSpread({}, state.fields), {}, {
          [name]: nextField
        });
        return _objectSpread({
          fields: nextFields,
          isFieldsTouched: true
        }, (0, _helpers.checkValidFieldsAndForm)(nextFields, state.invalidFields));
      }, () => {
        this.triggerOnChangeEvents({
          name
        });
      });
    };
    this.triggerOnChangeEvents = _ref4 => {
      let {
        name
      } = _ref4;
      const {
        onChangeFields: formBuilderOnChangeFields
      } = this.props;
      const nextFields = this.state.fields;

      // subscriptions from FromBuilder props
      (0, _helpers.callSubscriptions)({
        onChangeCallback: formBuilderOnChangeFields,
        nextFields,
        name
      });
      const internalsOnChanges = [...this.internalsOnChanges.values()];
      internalsOnChanges.forEach(onChangeCallback => (0, _helpers.callSubscriptions)({
        onChangeCallback,
        nextFields,
        name
      }));
    };
    this.formContextApiValue = {
      setInternalOnChanges: this.setInternalOnChanges,
      unsetInternalOnChanges: this.unsetInternalOnChanges,
      // external FromBuilder API
      setFields: this.setFields,
      getFields: this.getFields,
      setFieldsValue: this.setFieldsValue,
      getFieldsValue: this.getFieldsValue,
      getInitialValues: this.getInitialValues
    };
  }
  static getDerivedStateFromProps(props, state) {
    const nextState = {};
    if (props.errors && props.errors !== state.mirroredErrors) {
      nextState.mirroredErrors = props.errors;
      nextState.fields = (0, _helpers.setNewFieldsErrors)(props.errors, state.fields);
    }
    if (props.extraFieldsProps && props.extraFieldsProps !== state.mirroredExtraFieldsProps) {
      nextState.mirroredExtraFieldsProps = props.extraFieldsProps;
      nextState.fields = (0, _helpers.setExtraFieldsProps)(props.extraFieldsProps, state.fields);
    }
    return nextState;
  }

  // /* helpers for external API

  // */

  render() {
    const {
      withForm,
      renderForm = withForm ? _Form.default : _ref5 => {
        let {
          children: childrenProp
        } = _ref5;
        return childrenProp;
      },
      children
    } = this.props;
    return /*#__PURE__*/_react.default.createElement(FormContext.Provider, {
      value: {
        registerField: this.registerField,
        unregister: this.unregister,
        onChange: this.onChange,
        onSubmit: this.onSubmit,
        fields: this.state.fields,
        isFieldsTouched: this.state.isFieldsTouched
      }
    }, /*#__PURE__*/_react.default.createElement(FormContextApi.Provider, {
      value: this.formContextApiValue
    }, renderForm({
      onSubmit: this.onSubmit,
      children
    })));
  }
}
exports.default = FormBuilder;
FormBuilder.defaultProps = {
  validateOnBlur: false
};