"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = _interopRequireWildcard(require("react"));
var _constants = require("../constants");
var _FormBuilder = require("../FormBuilder");
var _helpers = require("../helpers");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && Object.prototype.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
const FormItemContext = /*#__PURE__*/_react.default.createContext({});
const WrapperItem = props => {
  const context = (0, _react.useContext)(_FormBuilder.FormContext);
  const formItemContext = (0, _react.useContext)(FormItemContext);
  if (!context.fields) {
    throw new Error('The FormItem must be inside the Form');
  }
  const {
    name: parentName
  } = formItemContext;
  const {
    registerField,
    unregister,
    onChange,
    fields
  } = context;
  const {
    name: localName
  } = props;
  const localProps = (0, _react.useMemo)(() => {
    const extraPropsLocal = {
      name: localName
    };
    if (parentName) {
      extraPropsLocal.name = "".concat(parentName, ".").concat(localName);
    }
    return extraPropsLocal;
  }, [parentName, localName]);
  return /*#__PURE__*/_react.default.createElement(FormItemMemo, _extends({}, props, localProps, {
    field: fields[localProps.name],
    onChange: onChange,
    registerField: registerField,
    unregister: unregister
  }));
};
const FormItem = props => {
  const {
      component: Component,
      formatter,
      validate,
      validateOnBlur,
      registerField,
      unregister,
      onChange: onChangeFromContext,
      field = {
        value: props.initialValue,
        error: null
      },
      errorMessage,
      name,
      initialValue,
      children
    } = props,
    nonServiceOwnProps = _objectWithoutProperties(props, ["component", "formatter", "validate", "validateOnBlur", "registerField", "unregister", "onChange", "field", "errorMessage", "name", "initialValue", "children"]);
  (0, _react.useEffect)(() => {
    if (Component) {
      registerField({
        componentProps: nonServiceOwnProps,
        name,
        value: initialValue,
        validate,
        errorMessage
      });
      return () => {
        unregister({
          name
        });
      };
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  const onChange = (0, _react.useCallback)(function (newValue) {
    let onBlurCall = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    const formattedValue = formatter ? formatter(newValue) : newValue;
    let error;
    if (!validateOnBlur || onBlurCall) {
      error = (0, _helpers.callValidateFunctions)({
        validate,
        value: formattedValue,
        errorMessage
      });
    }
    onChangeFromContext({
      name,
      value: formattedValue,
      error
    });
  }, [onChangeFromContext, name, formatter, errorMessage, validate, validateOnBlur]);
  const onBlur = (0, _react.useCallback)(() => {
    onChange(field.value, true);
  }, [field.value, onChange]);
  const valueContext = (0, _react.useMemo)(() => {
    return {
      name
    };
  }, [name]);
  const value = (0, _react.useMemo)(() => {
    return _constants.UNCONTROLLED_VALUES.includes(field.value) ? '' : field.value;
  }, [field.value]);
  return /*#__PURE__*/_react.default.createElement(FormItemContext.Provider, {
    value: valueContext
  }, Component ? /*#__PURE__*/_react.default.createElement(Component, _extends({}, field.extraFieldProps, nonServiceOwnProps, {
    name: name,
    value: value,
    onChange: onChange,
    error: field.error,
    onBlur: validateOnBlur ? onBlur : undefined
  })) : children);
};
FormItem.defaultProps = {
  errorMessage: 'Error'
};
const FormItemMemo = /*#__PURE__*/_react.default.memo(FormItem);
var _default = exports.default = WrapperItem;