"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.callSubscriptions = void 0;
exports.callValidateFunctions = callValidateFunctions;
exports.setNewFieldsErrors = exports.setExtraFieldsProps = exports.setExtraFieldProps = exports.checkValidFieldsAndForm = exports.checkUnTouchedFields = void 0;
var _Validators = _interopRequireDefault(require("./Validators"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : String(i); }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
const checkUnTouchedFields = fields => {
  const fieldsToSubmit = {};
  let fieldsWithError = null;
  let validForm = true;
  Object.keys(fields).forEach(name => {
    const field = fields[name];
    fieldsToSubmit[name] = field.value;
    if (!field.touched) {
      const {
        validate,
        value,
        errorMessage
      } = field;
      const error = callValidateFunctions({
        validate,
        value,
        errorMessage
      });
      if (checkInvalidField(error)) {
        validForm = false;
        // initiate fieldsWithError
        if (!fieldsWithError) {
          fieldsWithError = {};
        }
        fieldsWithError[name] = _objectSpread(_objectSpread({}, field), {}, {
          error
        });
      }
    }
    return;
  });
  return {
    fieldsToSubmit,
    fieldsWithError,
    validFormAfterValidateUntouchedFields: validForm
  };
};
exports.checkUnTouchedFields = checkUnTouchedFields;
const checkInvalidField = error => {
  // errorMessage prop isn't requirable
  // so it can be empty string
  if (error !== undefined && error !== null) {
    return true;
  }
  return false;
};
const checkValidFieldsAndForm = (nextFields, invalidFields) => {
  const nextInvalidFields = new Set(invalidFields);
  Object.keys(nextFields).forEach(name => {
    if (checkInvalidField(nextFields[name].error)) {
      return nextInvalidFields.add(name);
    }
    return nextInvalidFields.delete(name);
  });
  return {
    invalidFields: nextInvalidFields,
    valid: !nextInvalidFields.size
  };
};
exports.checkValidFieldsAndForm = checkValidFieldsAndForm;
function callValidateFunctions(_ref) {
  let {
    value,
    errorMessage = '',
    validate
  } = _ref;
  if (!validate) {
    return null;
  }
  if (Array.isArray(validate)) {
    return _Validators.default.composeValidators(validate)(value, errorMessage);
  }
  return validate(value, errorMessage);
}
const callSubscriptions = _ref2 => {
  let {
    onChangeCallback,
    nextFields,
    name
  } = _ref2;
  if (!onChangeCallback) {
    return;
  }
  const changedFields = {};
  if (Array.isArray(name)) {
    name.forEach(n => {
      changedFields[n] = nextFields[n];
    });
  } else {
    changedFields[name] = nextFields[name];
  }
  if (typeof onChangeCallback === 'function') {
    onChangeCallback(nextFields, changedFields);
    return;
  }
  const callChangeCallbackByName = name => {
    const onChangeCallbackPerField = onChangeCallback[name];
    if (onChangeCallbackPerField) {
      onChangeCallbackPerField(nextFields[name], nextFields);
    }
  };
  if (Array.isArray(name)) {
    name.forEach(n => {
      callChangeCallbackByName(n);
    });
  } else {
    callChangeCallbackByName(name);
  }
};
exports.callSubscriptions = callSubscriptions;
const setNewFieldsErrors = (errors, fields) => {
  const nextFields = _objectSpread({}, fields);
  if (!errors) {
    return nextFields;
  }
  Object.keys(errors).forEach(fieldKey => {
    nextFields[fieldKey] = _objectSpread(_objectSpread({}, fields[fieldKey]), {}, {
      error: errors[fieldKey]
    });
  });
  return nextFields;
};
exports.setNewFieldsErrors = setNewFieldsErrors;
const setExtraFieldsProps = (extraFieldsProps, fields) => {
  const nextFields = _objectSpread({}, fields);
  if (!extraFieldsProps) {
    return nextFields;
  }

  // Prepare only fields that already exist in the form
  // It doesn't miss configs set before mounting field,
  // Because registerField will call the setExtraFieldProps
  Object.keys(nextFields).forEach(fieldKey => {
    nextFields[fieldKey] = setExtraFieldProps(extraFieldsProps, nextFields[fieldKey]);
  });
  return nextFields;
};
exports.setExtraFieldsProps = setExtraFieldsProps;
const setExtraFieldProps = function () {
  let extraFieldsProps = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  let field = arguments.length > 1 ? arguments[1] : undefined;
  const commonGlobalConfig = extraFieldsProps.$all;
  const _ref3 = (extraFieldsProps === null || extraFieldsProps === void 0 ? void 0 : extraFieldsProps[field.name]) || {},
    {
      value
    } = _ref3,
    nonServiceExtraFieldsProps = _objectWithoutProperties(_ref3, ["value"]);
  return _objectSpread(_objectSpread({}, field), {}, {
    value: value !== null && value !== undefined ? value : field === null || field === void 0 ? void 0 : field.value,
    extraFieldProps: _objectSpread({
      commonGlobalConfig
    }, nonServiceExtraFieldsProps)
  });
};
exports.setExtraFieldProps = setExtraFieldProps;