import { FocusEvent } from 'react';
import { ValidatorType } from './Validators';
export declare type PickPropType<T, K extends keyof T> = T[K];
export declare type FieldNameType = PickPropType<IField, 'name'>;
export declare type ErrorsType = Record<FieldNameType, string> | null;
export declare type OnChangeFieldsType = ((allFields: Record<FieldNameType, IField>, updatedField: Record<FieldNameType, IField>) => void) | Record<FieldNameType, (field: IField, allFields: IFields) => void>;
export declare type FieldsWithErrorType = IFields | null;
declare type InitialValuesType = Record<FieldNameType, any>;
export declare type FormBuilderPropTypes = {
    errors?: ErrorsType;
    withForm?: boolean;
    renderForm?: Function;
    children?: React.ReactNode;
    onSubmit: (values: IFieldsToSubmit, fieldsWithError: FieldsWithErrorType) => Promise<any> | void;
    onChangeFields?: OnChangeFieldsType;
    validateOnBlur?: boolean;
    initialValues?: InitialValuesType;
    extraFieldsProps?: Record<FieldNameType, any>;
};
export declare type StateTypes = {
    isFieldsTouched: boolean;
    fields: IFields;
    invalidFields: Set<string>;
    valid: boolean;
    submitting: boolean;
    mirroredErrors: PickPropType<FormBuilderPropTypes, 'errors'>;
    mirroredExtraFieldsProps?: PickPropType<FormBuilderPropTypes, 'extraFieldsProps'>;
};
declare type SetFieldsOptionsType = {
    changeEvent: boolean;
};
export declare type GetFieldsType = (fieldKey?: FieldNameType) => IField | IFields;
export declare type SetFieldsType = (updates: Record<FieldNameType, Partial<IField>>, options?: SetFieldsOptionsType) => void;
export declare type SetFieldsValueType = (updates: Record<FieldNameType, any>, options?: SetFieldsOptionsType) => void;
export declare type GetFieldsValueType = (fieldKey?: FieldNameType) => IFieldsToSubmit | PickPropType<IField, 'value'>;
export declare type GetInitialValuesType = (fieldKey?: FieldNameType) => any;
export interface IFormContext {
    registerField: (field: RegisterFieldType) => void;
    unregister: (args: {
        name: string;
    }) => void;
    onChange: (field: OnChangeType) => void;
    onSubmit: (event: any) => void;
    fields: IFields;
    isFieldsTouched: boolean;
}
export interface IFormContextApi {
    setInternalOnChanges: Function;
    unsetInternalOnChanges: Function;
    getFields: GetFieldsType;
    setFields: SetFieldsType;
    setFieldsValue: SetFieldsValueType;
    getFieldsValue: GetFieldsValueType;
    getInitialValues: GetInitialValuesType;
}
export declare type IFieldsToSubmit = Record<FieldNameType, PickPropType<IField, 'value'>>;
export interface IFields {
    [name: string]: IField;
}
export interface IField {
    name: string;
    value?: any;
    error?: any;
    validate?: ValidateType;
    touched?: boolean;
    errorMessage?: ErrorMessageType;
    extraFieldProps?: any;
}
export declare type OnChangeType = {
    name: string;
    value: any;
    error?: any;
};
export declare type RegisterFieldType = {
    name: string;
    value: any;
    validate?: ValidateType;
    errorMessage?: ErrorMessageType;
    error?: any;
    componentProps: object;
};
export declare type ValidateType = ValidatorType<any> | Array<ValidatorType<any>>;
export declare type ErrorMessageType = string | string[];
declare type ComponentBasePropTypes = {
    name: string;
    value: any;
    onChange: (value: string) => void;
    error: any;
    onBlur?: (event: FocusEvent<any>) => void;
    commonGlobalConfig: any;
};
export declare type ComponentPropTypes<T> = React.ComponentType<T & ComponentBasePropTypes>;
export declare type CallValidateFunctionsType = {
    value: any;
    errorMessage?: string | string[];
    validate?: ValidatorType<any> | Array<ValidatorType<any>>;
};
export declare type CallSubscriptionsType = ({ onChangeCallback, nextFields, name, }: {
    onChangeCallback?: OnChangeFieldsType;
    nextFields: IFields;
    name: FieldNameType | FieldNameType[];
}) => void;
export {};
